# ## K-nearest neighbor Classification
#
# ### Import packages and data set


import numpy as np  # import `numpy` to support the computation of multi-dimensional arrays
import pandas as pd  # import `pandas` to manipulate table-liked formated data
import matplotlib.pyplot as plt  # import `matplotlib` to support data visualisation in Python
import seaborn as sns  # import `seaborn` to provide a domain-specific high-level interface for statistic data visualisation


#
# Read csv file named "Classified Data", the first column is used as label in the data frame


df = pd.read_csv("Classified Data.csv", index_col=0)


# print the first n rows in the DataFrame, n defaults to 5
print(df.head())


# print the summary of the DataFrame, for exmaple labels or data types
print(df.info())


# print the descriptive statistics, for example the mean value
print(df.describe())


# ### Check the spread of the features


# convert the DataFrame column labels object to py list named l
l = list(df.columns)
# print the names of features in list l, except the last one which is for target class
print(l[0 : len(l) - 1])

# **Run a 'for' loop to draw boxlots of all the features for '0' and '1' TARGET CLASS**
feature_fig, ax = plt.subplots(1, len(l) - 1, figsize=(16, 1.2), layout="constrained")
for i in range(len(l) - 1):  # for each feature
    # draw a boxplot for this feature, visualising the value destribution
    sns.boxplot(ax=ax[i], x="TARGET CLASS", y=l[i], data=df)

feature_fig.savefig("feature_distribution.png")


# ### Scale the features using sklearn.preprocessing package


from sklearn.preprocessing import StandardScaler  # import pkg to do standardizing

# instance a proprocessing class for standardizing
# StandardScaler standardrize the value to a normalized deviation from the mean
# i.e. z = (x - u) / s, where x: origin value, u: mean of samples, s: std deviation
scaler = StandardScaler()


# **Fit the features data only to this estimator (leaving the TARGET CLASS column) and transform**
#


# remove the class label column from the DataFrame, keep features only
# and compute the mean and std to prepare for later scaling
scaler.fit(df.drop("TARGET CLASS", axis=1))
# perform the standardrization on the feature DataFrame
scaled_features = scaler.transform(df.drop("TARGET CLASS", axis=1))

# put the standardrized feature into a new DataFrame,
# the labels for the new DataFrame(DF) is copied from the original DF
df_feat = pd.DataFrame(scaled_features, columns=df.columns[:-1])
# check the first 5 rows in the new feature DF
df_feat.head()


# ### Train/Test split, model fit and prediction


from sklearn.model_selection import (
    train_test_split,
)  # import pkg for spliting train/test dataset

X = df_feat  # Training data: feature (unused)
y = df["TARGET CLASS"]  # Training target: class label (unused)
# split the dataset, including feature and class label, into train set and test set,
# ratio of test set is 50%, random seed is 101, which can be used to generate the reproductive shuffling result
X_train, X_test, y_train, y_test = train_test_split(
    scaled_features, df["TARGET CLASS"], test_size=0.50, random_state=101
)


from sklearn.neighbors import (
    KNeighborsClassifier,
)  # import pkg for k-nearest neighbours classification

# instance a kNN classifier, the number of neighbours is 1
knn = KNeighborsClassifier(n_neighbors=1)


# fit the kNN classifier on the data, as the kNN classification is a "lazy learning" method,
# the model is not trained but just simply "remember" the training data
knn.fit(X_train, y_train)

# predict the class label of test set:
# kNN model compute the distance between the testing sample and training samples,
# and select top k samples whoes distance is the shortest,
# the argmax label of these samples is the prediction label
pred = knn.predict(X_test)


# **Evaluation of classification quality**


from sklearn.metrics import (
    classification_report,
    confusion_matrix,
)  # import pkg for evaluation

# compute the confusion matrix of classification,
# where C(i,j) is the number of samples that belong to class i, and predict as class j
# In a binary classification case like this task,
# C(0,0) is the count of true negatives (tn),  C(0,1) is the count of false positives (fp),
# C(1,0) is the count of false negatives (fn), C(1,1) is the count of true positives (tp).
conf_mat = confusion_matrix(y_test, pred)
# print the confusion matrix
print(conf_mat)


# print a text report of the classification result, includeing:
# precision: tp / (tp + fp), the ability of the classifier not to label a negative sample as positive.
# recall: tp / (tp + fn), the ability of the classifier to find all the positive samples.
# f1-score: 2 / (1/precision + 1/recall), predictive performance including both precision and recall
# support: The number of occurrences of each label in y_test
print(classification_report(y_test, pred))

# compute Misclassification error rate, i.e. the ratio that predict != ground truth, rounded to 3 decimals
print("Misclassification error rate:", round(np.mean(pred != y_test), 3))


# **Create a loop of different values for k and compute the errors**


# list to store error rate of kNN classifiers with differenct neighbour parameter k
error_rate = []
k_max = ...
for k in range(1, k_max):
    ...


# **Plot the errors**
